; vim:foldcolumn=0:autoindent:autochdir:scrolloff=0:nocursorline:nocursorcolumn:scrolljump=0:nostartofline:nowrap:nobackup:noundofile:noswapfile:virtualedit=:relativenumber:

; +---------------------------------------------------------------------------+
; |                                                                           |
; |  retroReaderUtils.ahk                                                     | 
; |  -------------------------------------------------------                  |
; |  Created: ~ mid apr 2018                                                  |
; |  Last Edited: n/a                                                         |
; |                                                                           |
; |  Script Purpose:                                                          |
; |  - utility functions for retroReader                                           |   
; |                                                                           |
; +---------------------------------------------------------------------------+


; crosshairs will not be repainted if passed old values same as current values
; note the parameters are for reference
; they are not what actually gets updated
UnpaintPaintCrosshairAsNecessary(n_oldCrosshairPaintDown_x, n_oldCrosshairPaintAcross_y)
{
	global n_crosshairPaintDown_x, n_crosshairPaintAcross_y

	if ((n_oldCrosshairPaintDown_x != n_crosshairPaintDown_x) || (n_oldCrosshairPaintAcross_y != n_crosshairPaintAcross_y))
	{
		Gui,1:Show,X%n_crosshairPaintDown_x% Y0
		Gui,2:Show,X0 Y%n_crosshairPaintAcross_y%
		mouseMovement_sethigh()
		; ^ note that this is doing exactly what it should do.
		;   if there is movement it is referring it to the mouseMovement_* routines
		;   so that it gets handled appropriately
	}
}

; this used to be separate a scanline would independently move down the screen
; now i've made it so it copies what the crosshair does
; so this routine is to also undraw the scanline when the crosshair moves
UnpaintPaintScanlineAsNecessary(n_oldCrosshairPaintDown_x, n_oldCrosshairPaintAcross_y)
{
	global n_crosshairPaintDown_x, n_crosshairPaintAcross_y, K_SCANHEIGHT, K_SCANWIDTH
	global K_SCANDISPLACEMENT_X, K_SCANDISPLACEMENT_Y

	if ((n_oldCrosshairPaintDown_x != n_crosshairPaintDown_x) || (n_oldCrosshairPaintAcross_y != n_crosshairPaintAcross_y))
	{
		n_tempXdisplacement := n_crosshairPaintDown_x+K_SCANDISPLACEMENT_X
		n_tempYdisplacement := n_crosshairPaintAcross_y+K_SCANDISPLACEMENT_Y
		Gui,4:Show,X%n_tempXdisplacement% Y%n_tempYdisplacement% H%K_SCANHEIGHT% W%K_SCANWIDTH%
		mouseMovement_sethigh()
		; ^ note that this is doing exactly what it should do.
		;   if there is movement it is referring it to the mouseMovement_* routines
		;   so that it gets handled appropriately
	}
}

; direction will not be used if speed is 0
MouseWheelToRightJoyConverter(cp1, cp1v, cp2, cp2v, n_mouseJoyWheelCounter, direction, speed)
{
	global K_GTE, K_GT, K_LTE, K_LT
	global K_TEXTFLOWINGUP, K_TEXTFLOWINGDOWN
	global b_scanCancel, b_crossCancel
	global bScrollStarted, b_scrollingNormal
	b_cp1 := 0
	b_cp2 := 0

	if (cp1=K_GT And n_mouseJoyWheelCounter>cp1v)
	{
		b_cp1 = 1
	}
	if (cp1=K_GTE And n_mouseJoyWheelCounter>=cp1v)
	{
		b_cp1 = 1
	}
	if (cp2=K_LT And n_mouseJoyWheelCounter<cp2v)
	{
		b_cp2 = 1
	}
	if (cp2=K_LTE And n_mouseJoyWheelCounter<=cp2v)
	{
		b_cp2 = 1
	}
	; COMPARISON 1: acting starting/continue moving
	if (b_cp1 And b_cp2 And 0!=speed)
	{
		; other parts of the code, e.g events like the mouse wheel
		; this is why there needs to exist a global flag
		;    remember status b4 ..
		CancelScanningIfStarted() ; <-- changes status of b_scanCancel
		CancelCrosshairIfActive() ; <-- ..and b_crossCancel
		if (b_crossCancel || b_scanCancel)
		{
			SetTitleMatchMode,2
			WinWaitActive,Reader,,2
			b_crossCancel = 0
			b_scanCancel = 0
		}
		if (Not bScrollStarted) ; you can assume it's activated
		{
			Send,^+h
			bScrollStarted := 1
		}
		; changed the order of this late mar 2018
		; And b_scrollingNormal, .... And Not b_scrollingNormal
		;not   And Not b_scrollingNormal, .... And b_scrollingNormal
		;together with b_scrollingNormal = 0, b_scrollingNormal = 1
		;not b_scrollingNormal = 1, b_scrollingNormal = 0
		; because this always seems to give rolling mouse wheel towards you
		; goes the opposite way of what  you'd expect to read i.e text flow backwards
		; which is opposite of the way normal reader works
		if (bScrollStarted And b_scrollingNormal And direction=K_TEXTFLOWINGUP)
		{
			Send,-
			b_scrollingNormal := 0
		}
		if (bScrollStarted And Not b_scrollingNormal And direction=K_TEXTFLOWINGDOWN)
		{
			Send,-
			b_scrollingNormal := 1
		}
		Send,%speed%
	}
	; COMPARISON 2: acting stopping
	; these two comparisons are mutually exclusive
	if (b_cp1 And b_cp2 And 0=speed)
	{
		if (bScrollStarted)
		{
			SetTitleMatchMode,2
			WinWaitActive,Reader,,2
			Send,^+h
			Sleep,150
			bScrollStarted := 0
		}
	}
}

progressScanLine()
{
	global n_scanlineClr
	global n_scanlineClrMax
	global n_BMWrotationCounter
	global h_aniGifRetro
	global n_anim
	global n_animsMax
	global hwnd_retro
	global n_animShowCountdown
	global n_animShowCountdownMax
	global h_aniGifAnnoyAdviceText
	global b_annoyAdviceOn
	global b_animShuffleHoldOn
	global K_SCANMIDDLEOFLEFTHAND

	n_animShowCountdown -= 1
	if (n_animShowCountdown <= 0)
	{
		if (!b_animShuffleHoldOn)
		; ^^ this line basically prevents the animation 'renewing' when the scan line
		; reaches the middle if desired, as well as the BMW panels
		; it does not prevent the changing of the color of line
		{
			n_animShowCountdown := n_animShowCountdownMax
			AniGif_DestroyControl(h_aniGifRetro) 
			if (0=n_anim)
			{
				n_animWidth = 85
				n_animStartX := K_SCANMIDDLEOFLEFTHAND-(n_animWidth//2)
				h_aniGifRetro := AniGif_CreateControl(hwnd_retro, n_animStartX, 1, n_animWidth, 18, "center")
				AniGif_LoadGifFromFile(h_aniGifRetro, "monsters.gif")
			}
			if (1=n_anim)
			{
				n_animWidth = 160
				n_animStartX := K_SCANMIDDLEOFLEFTHAND-(n_animWidth//2)
				h_aniGifRetro := AniGif_CreateControl(hwnd_retro, n_animStartX, 0, n_animWidth, 21, "center")
				AniGif_LoadGifFromFile(h_aniGifRetro, "snapper.gif")
			}
			if (2=n_anim)
			{
				n_animWidth = 80
				n_animStartX := K_SCANMIDDLEOFLEFTHAND-(n_animWidth//2)
				h_aniGifRetro := AniGif_CreateControl(hwnd_retro, n_animStartX, 0, n_animWidth, 21, "center")
				AniGif_LoadGifFromFile(h_aniGifRetro, "arcadians.gif")
			}
			if (3=n_anim)
			{
				n_animWidth = 140
				n_animStartX := K_SCANMIDDLEOFLEFTHAND-(n_animWidth//2)
				h_aniGifRetro := AniGif_CreateControl(hwnd_retro, n_animStartX, 0, n_animWidth, 21, "center")
				AniGif_LoadGifFromFile(h_aniGifRetro, "hopper.gif")
			}
			if (4=n_anim)
			{
				n_animWidth = 100
				n_animStartX := K_SCANMIDDLEOFLEFTHAND-(n_animWidth//2)
				h_aniGifRetro := AniGif_CreateControl(hwnd_retro, n_animStartX, 1, n_animWidth, 18, "center")
				AniGif_LoadGifFromFile(h_aniGifRetro, "killa.gif")
			}
			if (5=n_anim)
			{
				n_animWidth = 63
				n_animStartX := K_SCANMIDDLEOFLEFTHAND-(n_animWidth//2)
				h_aniGifRetro := AniGif_CreateControl(hwnd_retro, n_animStartX, 1, n_animWidth, 18, "center")
				AniGif_LoadGifFromFile(h_aniGifRetro, "blagger.gif")
			}
			if (6=n_anim)
			{
				n_animWidth = 215
				n_animStartX := K_SCANMIDDLEOFLEFTHAND-(n_animWidth//2)
				h_aniGifRetro := AniGif_CreateControl(hwnd_retro, n_animStartX, 0, n_animWidth, 20, "center")
				AniGif_LoadGifFromFile(h_aniGifRetro, "rocket.gif")
			}
			if (7=n_anim)
			{
				n_animWidth = 126
				n_animStartX := K_SCANMIDDLEOFLEFTHAND-(n_animWidth//2)
				h_aniGifRetro := AniGif_CreateControl(hwnd_retro, n_animStartX, 0, n_animWidth, 21, "center")
				AniGif_LoadGifFromFile(h_aniGifRetro, "exile.gif")
			}
			if (8=n_anim)
			{
				n_animWidth = 100
				n_animStartX := K_SCANMIDDLEOFLEFTHAND-(n_animWidth//2)
				h_aniGifRetro := AniGif_CreateControl(hwnd_retro, n_animStartX, 0, n_animWidth, 21, "center")
				AniGif_LoadGifFromFile(h_aniGifRetro, "fire.gif")
			}
			if (9=n_anim)
			{
				n_animWidth = 160
				n_animStartX := K_SCANMIDDLEOFLEFTHAND-(n_animWidth//2)
				h_aniGifRetro := AniGif_CreateControl(hwnd_retro, n_animStartX, 0, n_animWidth, 21, "center")
				AniGif_LoadGifFromFile(h_aniGifRetro, "repton.gif")
			}
			if (10=n_anim)
			{
				n_animWidth = 95
				n_animStartX := K_SCANMIDDLEOFLEFTHAND-(n_animWidth//2)
				h_aniGifRetro := AniGif_CreateControl(hwnd_retro, n_animStartX, 0, n_animWidth, 21, "center")
				AniGif_LoadGifFromFile(h_aniGifRetro, "citadel.gif")
			}
			if (11=n_anim)
			{
				n_animWidth = 160
				n_animStartX := K_SCANMIDDLEOFLEFTHAND-(n_animWidth//2)
				h_aniGifRetro := AniGif_CreateControl(hwnd_retro, n_animStartX, 0, n_animWidth, 21, "center")
				AniGif_LoadGifFromFile(h_aniGifRetro, "castlequest.gif")
			}
			if (12=n_anim)
			{
				n_animWidth = 150
				n_animStartX := K_SCANMIDDLEOFLEFTHAND-(n_animWidth//2)
				h_aniGifRetro := AniGif_CreateControl(hwnd_retro, n_animStartX, 0, n_animWidth, 21, "center")
				AniGif_LoadGifFromFile(h_aniGifRetro, "imogen.gif")
			}
			if (13=n_anim)
			{
				n_animWidth = 105
				n_animStartX := K_SCANMIDDLEOFLEFTHAND-(n_animWidth//2)
				h_aniGifRetro := AniGif_CreateControl(hwnd_retro, n_animStartX, 0, n_animWidth, 21, "center")
				AniGif_LoadGifFromFile(h_aniGifRetro, "revs.gif")
			}
			if (14=n_anim)
			{
				n_animWidth = 85
				n_animStartX := K_SCANMIDDLEOFLEFTHAND-(n_animWidth//2)
				h_aniGifRetro := AniGif_CreateControl(hwnd_retro, n_animStartX, 0, n_animWidth, 21, "center")
				AniGif_LoadGifFromFile(h_aniGifRetro, "raven.gif")
			}
			if (15=n_anim)
			{
				n_animWidth = 138
				n_animStartX := K_SCANMIDDLEOFLEFTHAND-(n_animWidth//2)
				h_aniGifRetro := AniGif_CreateControl(hwnd_retro, n_animStartX, 0, n_animWidth, 21, "center")
				AniGif_LoadGifFromFile(h_aniGifRetro, "strykers.gif")
			}
			if (16=n_anim)
			{
				n_animWidth = 130
				n_animStartX := K_SCANMIDDLEOFLEFTHAND-(n_animWidth//2)
				h_aniGifRetro := AniGif_CreateControl(hwnd_retro, n_animStartX, 0, n_animWidth, 21, "center")
				AniGif_LoadGifFromFile(h_aniGifRetro, "simcity.gif")
			}
			if (17=n_anim)
			{
				n_animWidth = 95
				n_animStartX := K_SCANMIDDLEOFLEFTHAND-(n_animWidth//2)
				h_aniGifRetro := AniGif_CreateControl(hwnd_retro, n_animStartX, 0, n_animWidth, 21, "center")
				AniGif_LoadGifFromFile(h_aniGifRetro, "frak.gif")
			}
			if (18=n_anim)
			{
				n_animWidth = 160
				n_animStartX := K_SCANMIDDLEOFLEFTHAND-(n_animWidth//2)
				h_aniGifRetro := AniGif_CreateControl(hwnd_retro, n_animStartX, 0, n_animWidth, 21, "center")
				AniGif_LoadGifFromFile(h_aniGifRetro, "galaforce.gif")
			}
			if (19=n_anim)
			{
				n_animWidth = 160
				n_animStartX := K_SCANMIDDLEOFLEFTHAND-(n_animWidth//2)
				h_aniGifRetro := AniGif_CreateControl(hwnd_retro, n_animStartX, 0, n_animWidth, 21, "center")
				AniGif_LoadGifFromFile(h_aniGifRetro, "thrust.gif")
			}
			if (20=n_anim)
			{
				n_animWidth = 85
				n_animStartX := K_SCANMIDDLEOFLEFTHAND-(n_animWidth//2)
				h_aniGifRetro := AniGif_CreateControl(hwnd_retro, n_animStartX, 0, n_animWidth, 21, "center")
				AniGif_LoadGifFromFile(h_aniGifRetro, "palace.gif")
			}
			if (21=n_anim)
			{
				n_animWidth = 180
				n_animStartX := K_SCANMIDDLEOFLEFTHAND-(n_animWidth//2)
				h_aniGifRetro := AniGif_CreateControl(hwnd_retro, n_animStartX, 0, n_animWidth, 21, "center")
				AniGif_LoadGifFromFile(h_aniGifRetro, "starswarm.gif")
			}
			if (22=n_anim)
			{
				n_animWidth = 90
				n_animStartX := K_SCANMIDDLEOFLEFTHAND-(n_animWidth//2)
				h_aniGifRetro := AniGif_CreateControl(hwnd_retro, n_animStartX, 0, n_animWidth, 21, "center")
				AniGif_LoadGifFromFile(h_aniGifRetro, "cliff.gif")
			}
			if (23=n_anim)
			{
				n_animWidth = 90
				n_animStartX := K_SCANMIDDLEOFLEFTHAND-(n_animWidth//2)
				h_aniGifRetro := AniGif_CreateControl(hwnd_retro, n_animStartX, 0, n_animWidth, 21, "center")
				AniGif_LoadGifFromFile(h_aniGifRetro, "cuthbert.gif")
			}
			if (24=n_anim)
			{
				n_animWidth = 128
				n_animStartX := K_SCANMIDDLEOFLEFTHAND-(n_animWidth//2)
				h_aniGifRetro := AniGif_CreateControl(hwnd_retro, n_animStartX, 0, n_animWidth, 21, "center")
				AniGif_LoadGifFromFile(h_aniGifRetro, "great.gif")
			}
			if (25=n_anim)
			{
				n_animWidth = 74
				n_animStartX := K_SCANMIDDLEOFLEFTHAND-(n_animWidth//2)
				h_aniGifRetro := AniGif_CreateControl(hwnd_retro, n_animStartX, 0, n_animWidth, 21, "center")
				AniGif_LoadGifFromFile(h_aniGifRetro, "imposs.gif")
			}
			if (26=n_anim)
			{
				n_animWidth = 100
				n_animStartX := K_SCANMIDDLEOFLEFTHAND-(n_animWidth//2)
				h_aniGifRetro := AniGif_CreateControl(hwnd_retro, n_animStartX, 0, n_animWidth, 21, "center")
				AniGif_LoadGifFromFile(h_aniGifRetro, "bubble.gif")
			}
			if (27=n_anim)
			{
				n_animWidth = 102
				n_animStartX := K_SCANMIDDLEOFLEFTHAND-(n_animWidth//2)
				h_aniGifRetro := AniGif_CreateControl(hwnd_retro, n_animStartX, 0, n_animWidth, 21, "center")
				AniGif_LoadGifFromFile(h_aniGifRetro, "karate.gif")
			}
			if (28=n_anim)
			{
				n_animWidth = 75
				n_animStartX := K_SCANMIDDLEOFLEFTHAND-(n_animWidth//2)
				h_aniGifRetro := AniGif_CreateControl(hwnd_retro, n_animStartX, 0, n_animWidth, 21, "center")
				AniGif_LoadGifFromFile(h_aniGifRetro, "turrican.gif")
			}
			if (29=n_anim)
			{
				n_animWidth = 63
				n_animStartX := K_SCANMIDDLEOFLEFTHAND-(n_animWidth//2)
				h_aniGifRetro := AniGif_CreateControl(hwnd_retro, n_animStartX, 0, n_animWidth, 21, "center")
				AniGif_LoadGifFromFile(h_aniGifRetro, "ninja.gif")
			}
			n_anim := PickOutNextRandom()
			experta_SetupAndShowBMWPanels()
		}
		n_scanlineClr := Mod(n_scanlineClr+1, n_scanlineClrMax)
		s_scanLineClrCode := getScanLineClrCodeStupidlyInverted(n_scanlineClr)
		s_scanLineClrCodewith0x = 0x%s_scanLineClrCode%
		AniGif_SetBkColor(h_aniGifAnnoyAdviceText, s_scanLineClrCodewith0x)
		changeTextAndLineColours()
		n_BMWrotationCounter += 1
	}
	GuiControl,4:,MyText,%A_YYYY% %A_MMM% %A_DD% %A_Hour%:%A_Min% of a %A_DDDD%
}
; progressScanLine()

getScanLineClrCode(n_scanlineClr)
{
	result = ""
	if (0=n_scanlineClr)
		result=888888
	if (1=n_scanlineClr)
		result=880000
	if (2=n_scanlineClr)
		result=44DD00
	if (3=n_scanlineClr)
		result=FFDD00
	if (4=n_scanlineClr)
		result=FF8800
	if (5=n_scanlineClr)
		result=00DDDD
	if (6=n_scanlineClr)
		result=BBBBBB
	return result
}

; because aniGif SetBkColor inverts the order
; RGB = BGR
getScanLineClrCodeStupidlyInverted(n_scanlineClr)
{
	result = ""
	if (0=n_scanlineClr)
		result=888888
	if (1=n_scanlineClr)
		result=000088
	if (2=n_scanlineClr)
		result=0044DD
	if (3=n_scanlineClr)
		result=00DDFF
	if (4=n_scanlineClr)
		result=0088FF
	if (5=n_scanlineClr)
		result=DDDD00
	if (6=n_scanlineClr)
		result=BBBBBB
	return result
}

getScanLineClrTxCode(n_scanlineClr)
{
	result = ""
	If (0=n_scanlineClr)
		result=White
	If (1=n_scanlineClr)
		result=Red
	If (2=n_scanlineClr)
		result=0000EE
	If (3=n_scanlineClr)
		result=00CC00
	If (4=n_scanlineClr)
		result=White
	If (5=n_scanlineClr)
		result=cBlack
	If (6=n_scanlineClr)
		result=White
	return result
}

changeTextAndLineColours()
{
	global n_scanlineClr
	global h_aniGifRetro

	s_scanLineClrCode := getScanLineClrCode(n_scanlineClr)
	s_scanLineClrCodewith0x = 0x%getScanLineClrCode%
	s_scanLineTxClrCode := getScanLineClrTxCode(n_scanlineClr)
	;                      ^^ note you don't need to stick in 0x in here
	;                         it will work as-is
	Gui,4:Color,%s_scanLineClrCode%
	Gui,4:Font,s11 c%s_scanLineTxClrCode%,Arial 
	GuiControl,4:Font,MyText
	AniGif_SetBkColor(h_aniGifRetro, s_scanLineClrCodewith0x)
}

; note height and width mean panel
showBMWPanels(midStartX,midStartY,height,width,n_LorRpanel1or2, counter)
{
	if (1=n_LorRpanel1or2)
	{
		n_LorRpanel1or2 := 5
	}
	else
	{
		n_LorRpanel1or2 := 6
	}
	n_moverY := 0
	n_moverX := 0
	if (0=Mod(counter,4))
	{
		n_moverX := 0
		while (n_moverX <= midStartX)
		{
			Gui,%n_LorRpanel1or2%:Show,x%n_moverX% y0 h%height% w%width%
			n_moverX += 4
		}
		Gui,%n_LorRpanel1or2%:Show,x%midStartX% y0 h%height% w%width%
	}
	if (1=Mod(counter,4))
	{
		n_moverY := 0
		while (n_moverY <= midStartY)
		{
			Gui,%n_LorRpanel1or2%:Show,x%midStartX% y%n_moverY% h%height% w%width%
			n_moverY += 4
		}
		Gui,%n_LorRpanel1or2%:Show,x%midStartX% y%midStartY% h%height% w%width%
	}
	if (2=Mod(counter,4))
	{
		n_moverX := midStartX
		while (n_moverX > 0)
		{
			Gui,%n_LorRpanel1or2%:Show,x%n_moverX% y%midStartY% h%height% w%width%
			n_moverX -= 4
		}
		Gui,%n_LorRpanel1or2%:Show,x%n_moverX% y%midStartY% h%height% w%width%
	}
	if (3=Mod(counter,4))
	{
		n_moverY := midStartY
		while (n_moverY >= 0)
		{
			Gui,%n_LorRpanel1or2%:Show,x0 y%n_moverY% h%height% w%width%
			n_moverY -= 4
		}
		Gui,%n_LorRpanel1or2%:Show,x0 y0 h%height% w%width%
	}
}

experta_SetupAndShowBMWPanels()
{
	global bmwpanel1_ID, bmwpanel2_ID
	global n_BMWrotationCounter

	height := A_ScreenHeight//2
	width := A_ScreenWidth//2
	bmwMidX := A_ScreenWidth//2
	bmwMidY := A_ScreenHeight//2
	counter := n_BMWrotationCounter
	showBMWPanels(bmwMidX,bmwMidY,height,width,1,counter)
	bmwMidX := width
	bmwMidY := height + 1
	counter := n_BMWrotationCounter+2
	showBMWPanels(bmwMidX,bmwMidY,height,width,2,counter)
}

doActuallyAskUserWhetherScrollingIsNormalOrNot()
{
	global b_wheelConfirmedUp, b_wheelConfirmedDown, b_calibrationProgress, b_scrollingNormal

	Sleep,500,
	Send,{Esc}
	Send,^+h
	Sleep,250
	Gui,configInput:New
	Gui,configInput:Add, Picture, w480 h240, calibrate2.png
	Gui,configInput:Color,FFFFFF
	Gui,configInput:Show,AutoSize Center
	WinSet, Style, -0xC00000, A

	; note that this bit performs the role of calibrating
	; |
	; v
	b_calibrationProgress = 1
	b_wheelConfirmedUp = 0
	b_wheelConfirmedDown = 0
	While (!b_wheelConfirmedUp && !b_wheelConfirmedDown)
	{
		Sleep, 250
	}
	Gui,configInput:Hide
	Gui,configInput:New
	Gui,configInput:Add, Picture, w480 h240, dontTouchArghh.png
	Gui,configInput:Color,FFFFFF
	Gui,configInput:Show,AutoSize Center
	WinSet, Style, -0xC00000, A
	Sleep,1400
	Gui,configInput:Hide
	if (b_wheelConfirmedUp)
	{
		b_scrollingNormal = 0
		; changed this late Mar 2018 to mean reverse normal scrolling 
		; i.e opposite flow to what you would normally read
	}
	else
	{
		b_scrollingNormal = 1
		; changed this late Mar 2018 to mean the normal
		; flow you would read down the document the normal way
	}
	Sleep,150
	Send,^+h
	return
}

calibrate()
{
	global b_calibrationIsActiveAndOngoing

	b_calibrationIsActiveAndOngoing = 1
	properlySwitchCrosshairOff()
	iWouldLikeScanningToStop()
	WinActivate,Reader
	doActuallyAskUserWhetherScrollingIsNormalOrNot()
	Sleep,1000
	Send,1
	ReminderOfKeys()
	b_calibrationIsActiveAndOngoing = 0
	return
}

ReminderOfKeys()
{
	global n_userBeingGivenInstruction

	Gui,3:Show
	; ^^ this is a yellow 'plastic cover' that goes over the whole screen
	Sleep,120
	; ^^ because it's big it's better to let the system catch up before ploughing ahead
	WinSet, Style, -0xC00000, A
	Sleep,60
	; ^^ and just to be on the safe side wait again for the system
	Gui,configInput:New
	Gui,configInput:Add, Picture, w480 h240, rememberTheKeys.png
	Gui,configInput:Color,FFFFFF
	Gui,configInput:+Hwndhwnd_theKeys
	h_aniGifTheKeys := AniGif_CreateControl(hwnd_theKeys, 20, 20, 480, 240, "center")
	AniGif_LoadGifFromFile(h_aniGifTheKeys, "rememberTheKeys.gif")
	;AniGif_SetBkColor(h_aniGifRetro, 0xFF00FF)
	Gui,configInput:Show,AutoSize Center
	WinSet, Style, -0xC00000, A
	WinSet, Region, 20-20 W480 H240 R30-30, A
	n_userBeingGivenInstruction = 20740 
	SetTimer,userInstructionLoop,80
}

userInstructionLoop()
{
	global n_userBeingGivenInstruction, b_spacePushedToForceTerminateAvatarGuy

	n_userBeingGivenInstruction-=80
	if (n_userBeingGivenInstruction<=0)
	{
		SetTimer,,Off
		; ^^ this subroutine will not be called no more
		n_userBeingGivenInstruction = 0
		Gui,configInput:Hide
		Gui,3:Hide
		if (b_spacePushedToForceTerminateAvatarGuy)
		{
			b_spacePushedToForceTerminateAvatarGuy = 0
		}
		else
		{
			; i'm gonna have it so if the user DOES NOT push avatar away
			; then they get the final 'good luck' message
			; otherwise they presumably can't be bother so everything
			; just gets cleared out of the way (if case above)
			; then again if they can be bother then it sounds like
			; they appreciate a little extra attention
			Gui,configInput:New
			Gui,configInput:Add, Picture, w480 h240, bestOfLuck.png
			Gui,configInput:Color,FFFFFF
			Gui,configInput:Show,AutoSize Center
			WinSet, Style, -0xC00000, A
			Sleep,1800
			Gui,configInput:Hide
		}
	}
}

maintainThatSomethingShouldHaveHappen(whatShouldHaveHappen)
{
	global b_scanCancel, b_crossCancel
	global b_crossApparentlyOffButReallyStillOnscreen, b_crossOn

	Gui,configInput:New
	Gui,configInput:Font,cRed s18,Arial
	; ^^ i wanna specify a font rather than leave the default one
	Gui,configInput:Add,Text,x80 y20,%whatShouldHaveHappen%
	; ^^ i'm trying to have it somewhere so it looks nice with margins to either side
	Gui,configInput:Color,FFFFFF
	Gui,configInput:Show,w400 h120 Center
	;                    ^^ I'm trying to make the size of the box as a whole large enough to accomodate
	WinSet, Style, -0xC00000, A
	WinSet, Region, 20-20 W480 H240 R30-30, A
	Sleep,1000
	Gui,configInput:Hide
}

CancelCrosshairIfActive() ; the assumption called only if scanning
{
	global b_crossCancel

	if (b_scanOn || b_crossApparentlyOffButReallyStillOnscreen)
	{
		properlySwitchCrosshairOff()
		b_crossCancel := 1
		Sleep,60
		; ^ this is important because we are changing the windows draw order
		; as a result of the cancellation, windows needs to update itself properly first
	}
}

CancelScanningIfStarted()
{
	global b_scanCancel, b_scanOn

	if (b_scanOn)
	{
		b_scanCancel := 1
		iWouldLikeScanningToStop()
	}
}

iWouldLikeScanningToStart()
{
	global b_scanOn, n_crosshairPaintDown_x, n_crosshairPaintAcross_y, K_SCANWIDTH, K_SCANHEIGHT
	global K_SCANDISPLACEMENT_X, K_SCANDISPLACEMENT_Y
	global b_crossApparentlyOffButReallyStillOnscreen
	global n_crossApparentlyOffButReallyStillOnscreen_x
	global n_crossApparentlyOffButReallyStillOnscreen_y
	static firstTime = 1

	if (!b_scanOn)
	{
		experta_SetupAndShowBMWPanels()
		if (b_crossApparentlyOffButReallyStillOnscreen)
		{
			; what this means is that the user's current position isn't really
			; applicable as the pseudo park behaviour of the cross hair is now on
			n_tempXdisplacement := n_crossApparentlyOffButReallyStillOnscreen_x+K_SCANDISPLACEMENT_X
			n_tempYdisplacement := n_crossApparentlyOffButReallyStillOnscreen_y+K_SCANDISPLACEMENT_Y
		}
		else
		{
			n_tempXdisplacement := n_crosshairPaintDown_x+K_SCANDISPLACEMENT_X
			n_tempYdisplacement := n_crosshairPaintAcross_y+K_SCANDISPLACEMENT_Y
		}
		if (firstTime)
		{
			Gui,4:Show,X%n_tempXdisplacement% Y%n_tempYdisplacement% H%K_SCANHEIGHT% W%K_SCANWIDTH%
			GuiControl,4:,MyText,SPACE = toggle taskbar
			firstTime := 0
		}
		else
		{
			Gui,4:Show,X%n_tempXdisplacement% Y%n_tempYdisplacement% H%K_SCANHEIGHT% W%K_SCANWIDTH%
			GuiControl,4:,MyText,%A_YYYY% %A_MMM% %A_DD% %A_Hour%:%A_Min% of a %A_DDDD%
		}
		b_scanOn := 1
	}
}

iWouldLikeScanningToStop()
{
	global b_scanOn
	b_scanOn := 0

	Gui,4:Hide
	;  (^^ this is the scan line panel)
	Sleep,30
	;^^ if you don't put Sleeps in  the system can leave a 'ghost'
	;
	Gui,5:Hide
	;  (^^ BMW Panel 1)
	Sleep,30
	;^^ if you don't put Sleeps in  the system can leave a 'ghost'
	;
	Gui,6:Hide
	;  (^^ BMW Panel 2)
	Sleep,30
	;^^ if you don't put Sleeps in  the system can leave a 'ghost'
	;
}

SeeIfMouseNeedsTurnOn()
{
	global b_crossOn, b_scanOn

	if (!b_crossOn And !b_scanOn)
	{
		SystemCursor("On")
	}
	else
	{
		SystemCursor("Off")
	}
	return
}

isInScrollbar()
{
	MouseGetPos, nMouseX, nMouseY
	if ((A_ScreenWidth-40<nMouseX))
	{
		return 1
	}
	else
	{
		return 0
	}
}

properlySwitchCrosshairOff()
{
	global b_crossOn, b_crossApparentlyOffButReallyStillOnscreen

	Sleep,80
	if (b_crossOn || b_crossApparentlyOffButReallyStillOnscreen)
	{
		;hide crosshair
		Gui,1:Hide
		Sleep,50
		Gui,2:Hide
		Sleep,50
		;^^ if you don't put Sleeps in  the system can leave a 'ghost' [EDIT: even 30,50 was not enough]
		;
		b_crossOn = 0
		; definitely mark the crosshair as well and truly gone.. !
		b_crossApparentlyOffButReallyStillOnscreen = 0
	}
}

IConsiderYouStill()
{
	global n_mouseJoyWheelCounter, n_mouseMovementConcern

	if (50=n_mouseJoyWheelCounter && 0=n_mouseMovementConcern)
	;                                ^^ notice the condition is not only has the reader
	;                                   scrolling stopped but also do you believe has the mouse
	;                                   itself stopped moving 0=
		return 1
	return 0
}

; note: there's a difference between
; mouse movement concern vs. mousejoy wheel counter
; mouse movement concern is any mouse activity = non zero positive (*)
; mousejoy wheel counter is the speed of the scrolling
; * slowly decrease to zero, this is quite important! (**)
mouseMovement_sethigh()
{
	global n_mouseMovementConcern

	n_mouseMovementConcern = 8
}

; note this also deals with the mouse movement concern, not scroll speed
; ** this is the function that decreases it
;    if you forget to call it it could be a bit problem
mouseMovement_countdown()
{
	global n_mouseMovementConcern

	n_mouseMovementConcern -= 1
	if (n_mouseMovementConcern<0)
		n_mouseMovementConcern = 0
}

; and note this does deal with speed of scroll, not mouse movement concern
tooMuchInTheMiddle()
{
	global n_mouseJoyWheelCounter

	if (n_mouseJoyWheelCounter>=42 || n_mouseJoyWheelCounter<=58)
		return 1
	return 0
}

GiveUpByLookingInBag()
{
	global a_shuffleRecord, n_animsMax
	n_it = 0

	while (1=a_shuffleRecord[n_it] And n_it<n_animsMax)
	{
		n_it += 1
	}
	if (n_it = n_animsMax)
	{
		n_it = 0
		while (n_it < n_animsMax)
		{
			a_shuffleRecord[n_it] = 0
			n_it += 1
		}
		Random, n_it, 0, %n_animsMax%
	}
	a_shuffleRecord[n_it] := 1
	return n_it
}

PickOutNextRandom()
{
	global a_shuffleRecord, n_animsMax
	n_maxTries = 8
	n_tries = 0
	n_thisThat = 0

	while (n_tries < n_maxTries)
	{
		Random, n_thisThat, 0, %n_animsMax%
		if (0 = a_shuffleRecord[n_thisThat])
		{
			a_shuffleRecord[n_thisThat] = 1
			return n_thisThat
		}
		n_tries += 1
	}
	n_thisThat := GiveUpByLookingInBag()
	return n_thisThat
}

notifyRightClickIsIffyByFlashing()
{
	global b_crossOn

	i = 0
	if (b_crossOn)
	{
		while (i<=1)
		{
			Gui,Color,00FF00
			Gui,2:Color,00FF00
			Sleep,220
			Gui,Color,FF0000
			Gui,2:Color,FF0000
			Sleep,220
			i++
		}
	}
}

notifyParkedByFlashing()
{
	global b_crossOn

	i = 0
	if (b_crossOn)
	{
		while (i<=1)
		{
			Gui,Color,FFFF00
			Gui,2:Color,FFFF00
			Sleep,220
			Gui,Color,FF0000
			Gui,2:Color,FF0000
			Sleep,220
			i++
		}
	}
}
