; vim:foldcolumn=0:autoindent:autochdir:scrolloff=0:nocursorline:nocursorcolumn:scrolljump=0:nostartofline:nowrap:nobackup:noundofile:noswapfile:virtualedit=all:

; +---------------------------------------------------------------------------+
; |                                                                           |
; |  retroReaderAniGif.ahk                                                    | 
; |  -------------------------------------------------------                  |
; |  Created: ~ mid Apr 2018                                                  |
; |  Last Edited: n/a                                                         |
; |                                                                           |
; |  Script Purpose:                                                          |
; |  - AniGif library wrappers used by retroReader                            |   
; |                                                                           |
; +---------------------------------------------------------------------------+

AniGif_CreateControl(_guiHwnd, _x, _y, _w, _h,_style=""){
	local hAniGif, agHwnd
	local msg, style 
	static $bFirstCall := true

	If ($bFirstCall)
	{
		$bFirstCall := false
		; It will be unloaded at script end
		hAniGif := DllCall("LoadLibrary", "Str", "AniGif.dll")
	}
	style := 0
	If (_style != "")
	{
		If _style contains autosize
			style |= 1	; WAGS_AUTOSIZE
		If _style contains center
			style |= 2	; WAGS_CENTER
		If _style contains hyperlink
			style |= 4	; WAGS_HYPERLINK
	}	
	style := 0x50000000 | style	
	agHwnd := DLLCall("CreateWindowEx"
			, "UInt", 0                     ; Style, can be WS_EX_CLIENTEDGE = 0x200
			, "Str", "AniGIF"               ; Class Name
			, "Str", "AnimatedGif"          ; Window name
			, "UInt",  style                ; Window style
			, "Int", _x                     ; X position
			, "Int", _y                     ; Y position
			, "Int", _w                     ; Width
			, "Int", _h                     ; Height
			, "UInt", _guiHwnd              ; Handle of parent
			, "UInt", 0                     ; Menu
			, "UInt", 0               ; hInstance of the module registering the component's class
			, "UInt", 0)                    ; User defined style

	If (ErrorLevel != 0 or agHwnd = 0)
	{
		msg = %msg% Cannot create AniGif control (%ErrorLevel%/%A_LastError%)
		Gosub AniGif_CreateControl_CleanUp
		Return msg
	}
	Return agHwnd

AniGif_CreateControl_CleanUp:	; In case of error	
Return
}

AniGif_DestroyControl(_agHwnd)
{
	If (_agHwnd != 0)
	{
		AniGif_UnloadGif(_agHwnd)
		DllCall("DestroyWindow", "UInt", _agHwnd)
	}
}

AniGif_LoadGifFromFile(_agHwnd, _gifFile)
{	
	VarSetCapacity(var, StrPut(_gifFile, "cp0") * 2)
	StrPut(_gifFile, &var, "cp0")
	SendMessage, 2024, 0, &var, , ahk_id %_agHwnd%
	; SendMessage 2024, 0, &_gifFile, , ahk_id %_agHwnd%
}

AniGif_UnloadGif(_agHwnd)
{	
	SendMessage 2026, 0, 0, , ahk_id %_agHwnd%
}

AniGif_SetHyperlink(_agHwnd, _url)
{	
	SendMessage 2027, 0, &_url, , ahk_id %_agHwnd%
}

AniGif_Zoom(_agHwnd, _bZoomIn)
{	
	PostMessage 2028, 0, _bZoomIn, , ahk_id %_agHwnd%
}

AniGif_SetBkColor(_agHwnd, _backColor)
{	
	PostMessage 2029, 0, _backColor, , ahk_id %_agHwnd%
}

SystemCursor(OnOff=1)   ; INIT = "I","Init"; OFF = 0,"Off"; TOGGLE = -1,"T","Toggle"; ON = others
{
    static AndMask, XorMask, $, h_cursor
        ,c0,c1,c2,c3,c4,c5,c6,c7,c8,c9,c10,c11,c12,c13 ; system cursors
        , b1,b2,b3,b4,b5,b6,b7,b8,b9,b10,b11,b12,b13   ; blank cursors
        , h1,h2,h3,h4,h5,h6,h7,h8,h9,h10,h11,h12,h13   ; handles of default cursors
    if (OnOff = "Init" or OnOff = "I" or $ = "")       ; init when requested or at first call
    {
        $ = h                                          ; active default cursors
        VarSetCapacity( h_cursor,4444, 1 )
        VarSetCapacity( AndMask, 32*4, 0xFF )
        VarSetCapacity( XorMask, 32*4, 0 )
        system_cursors = 32512,32513,32514,32515,32516,32642,32643,32644,32645,32646,32648,32649,32650
        StringSplit c, system_cursors, `,
        Loop %c0%
        {
            h_cursor   := DllCall( "LoadCursor", "Ptr",0, "Ptr",c%A_Index% )
            h%A_Index% := DllCall( "CopyImage", "Ptr",h_cursor, "UInt",2, "Int",0, "Int",0, "UInt",0 )
            b%A_Index% := DllCall( "CreateCursor", "Ptr",0, "Int",0, "Int",0
                , "Int",32, "Int",32, "Ptr",&AndMask, "Ptr",&XorMask )
        }
    }
    if (OnOff = 0 or OnOff = "Off" or $ = "h" and (OnOff < 0 or OnOff = "Toggle" or OnOff = "T"))
        $ = b  ; use blank cursors
    else
        $ = h  ; use the saved cursors

    Loop %c0%
    {
        h_cursor := DllCall( "CopyImage", "Ptr",%$%%A_Index%, "UInt",2, "Int",0, "Int",0, "UInt",0 )
        DllCall( "SetSystemCursor", "Ptr",h_cursor, "UInt",c%A_Index% )
    }
}
